package com.uas.cloud.mall.carousel.api;

import com.uas.cloud.base.commons.common.api.ResourceNotFoundException;
import com.uas.cloud.base.commons.common.api.ServiceError;
import com.uas.cloud.mall.carousel.domain.Carousel;
import com.uas.cloud.mall.carousel.service.CarouselService;
import java.net.URI;
import java.util.List;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.util.UriComponentsBuilder;

/**
 * 轮播服务API
 *
 * @author yangck
 * @create 2017-02-16 10:41
 */
@RestController
@RequestMapping("/carousels")
public class CarouselController {

    private final Logger logger = LogManager.getLogger(getClass());

    private final CarouselService carouselService;

    @Autowired
    public CarouselController(CarouselService carouselService) {
        this.carouselService = carouselService;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET, produces = "application/json")
    public Carousel get(@PathVariable Long id) {
        logger.info("/carousels, method: get, carousel: " + id);

        return carouselService.findOne(id);
    }

    @RequestMapping(method = RequestMethod.POST, consumes = "application/json")
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<Carousel> save(@RequestBody Carousel carousel, UriComponentsBuilder ucb) {
        logger.info("/carousel, method: post, usedFor: " + carousel.getUsedFor());

        Carousel saved = carouselService.save(carousel);

        HttpHeaders headers = new HttpHeaders();
        URI locationUri = ucb.path("/carousel")
                .path(String.valueOf(carousel.getId()))
                .build()
                .toUri();
        headers.setLocation(locationUri);

        return new ResponseEntity<>(saved, headers, HttpStatus.CREATED);
    }

    @RequestMapping(method = RequestMethod.PUT, consumes = "application/json")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<Carousel> update(@RequestBody Carousel carousel) {
        logger.info("/carousel, method: put, usedFor: " + carousel.getUsedFor());

        Carousel saved = carouselService.save(carousel);

        return new ResponseEntity<>(saved, HttpStatus.OK);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public void delete(@PathVariable Long id) {
        logger.info("/carousels, method: delete, carousel: " + id);

        carouselService.delete(id);
    }

    @RequestMapping(method = RequestMethod.GET, produces = "application/json")
    public List<Carousel> carousels(@RequestParam(value = "usedFor", defaultValue = "home") String usedFor) {
        logger.info("/carousel, get, usedFor: " + usedFor);

        return carouselService.findByUsedFor(usedFor);
    }

    @GetMapping(value = "/usedFor", produces = "application/json")
    public List<String> getUsedFor() {
        logger.info("/carousel/usedFor, get");

        return carouselService.getUsedFor();
    }

    @ExceptionHandler(ResourceNotFoundException.class)
    @ResponseStatus(HttpStatus.NOT_FOUND)
    public ServiceError resourceNotFound(ResourceNotFoundException e) {
        Long resourceId = e.getResourceId();

        return new ServiceError(4, "Carousel [" + resourceId + "] not found");
    }
}
